.swh-vault-table{border-bottom:none!important;margin-bottom:0!important}.swh-vault-table td{vertical-align:middle!important;border-top:1px solid #ddd!important}
/*# sourceMappingURL=vault.722168a018a7b8fa9ede.css.map */