{% extends "layout.html" %}

{% comment %}
Copyright (C) 2017-2018  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% block title %}Error {{ error_code }} &ndash; Software Heritage archive {% endblock %}

{% block navbar-content %}
<h4>Error</h4>
{% endblock %}

{% block content %}
{% include "includes/http-error.html" %}
{% endblock %}