{% extends "./browse.html" %}

{% comment %}
Copyright (C) 2017-2018  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% block swh-browse-content %}
{% include "includes/top-navigation.html" %}
{% include "includes/content-display.html" %}
{% endblock %}
