{% extends "./layout.html" %}

{% comment %}
Copyright (C) 2017-2020  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% load static %}

{% block navbar-content %}
<h4>Search archived software</h4>
{% endblock %}

{% block browse-content %}

{% include "includes/origin-search-form.html" %}
<hr>
<div id="swh-origin-search-results" class="mb-3" style="display: none;">
  <div class="table-responsive">
    <table class="table swh-table swh-table-striped" id="origin-search-results">
      <thead>
        <tr>
          <th>Origin type</th>
          <th>Origin url</th>
          <th>Archiving status</th>
        </tr>
      </thead>
      <tbody>
      </tbody>
    </table>
  </div>
</div>
<div class="swh-loading">
  <img src="{% static 'img/swh-spinner.gif' %}"></img>
  <p>Searching origins ...</p>
</div>
<p id="swh-no-result" style="display: none; white-space: pre;">
  <br/>
  No origins matching the search criteria were found.
</p>

<ul class="pagination justify-content-center swh-search-pagination">
  <li class="disabled page-item" id="origins-prev-results-button"><a class="page-link" href="#" tabindex="-1">Previous</a></li>
  <li class="disabled page-item" id="origins-next-results-button"><a class="page-link" href="#" tabindex="-1">Next</a></li>
</ul>

<script>
  swh.webapp.initPage('search');
</script>
{% endblock %}
