{% extends "layout.html" %}

{% comment %}
Copyright (C) 2017-2019  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% load swh_templatetags %}
{% load render_bundle from webpack_loader %}

{% block title %}{{ heading }} &ndash; Software Heritage archive {% endblock %}

{% block header %}
{% render_bundle 'browse' %}
{% render_bundle 'vault' %}
{% render_bundle 'save' %}
{% endblock %}

{% block content %}
{% block browse-content %}{% endblock %}
{% endblock %}
